#ifndef ALGORITHMFACTORY_H_
#define ALGORITHMFACTORY_H_

#include "PromptAlgorithm.h"
#include "ButtonAlgorithm.h"
#include "RandomAlgorithm.h"
#include "SmartAlgorithm.h"

using namespace std;

/**
 * Types of algorithms.
 */
enum class AlgorithmType { PROMPT, BUTTON, RANDOM, SMART };
const string ALGORITHM_STRINGS[] { "PROMPT", "BUTTON", "RANDOM", "SMART" };

/**
 * Maker of algorithms.
 */
class AlgorithmFactory
{
public:
    /**
     * Static factory function to create an algorithm object.
     */
    static Algorithm *make(const AlgorithmType which)
    {
        if      (which == AlgorithmType::PROMPT) return new PromptAlgorithm();
        else if (which == AlgorithmType::BUTTON) return new ButtonAlgorithm();
        else if (which == AlgorithmType::RANDOM) return new RandomAlgorithm();
        else                                     return new SmartAlgorithm();
    }
};

#endif /* ALGORITHMFACTORY_H_ */
