#ifndef CHOICE_H_
#define CHOICE_H_

#include <iostream>
#include <string>

using namespace std;

/**
 * What a player chooses each round.
 */
enum class Choice { ROCK, PAPER, SCISSORS, NONE };
const string CHOICE_STRINGS[] { "ROCK", "PAPER", "SCISSORS", "???" };
const char   CHOICE_CHARS[]   { 'R', 'P', 'S', '?' };

/**
 * Overloaded > operator for chosen objects. The greater one wins.
 * @param choice1 the first choice to compare.
 * @param choice2 the second choice to compare.
 * @return true if choice1 wins over choice2, else return false.
 */
inline bool operator >(const Choice choice1, const Choice choice2)
{
    return    ((choice1 == Choice::ROCK)     && (choice2 == Choice::SCISSORS))
           || ((choice1 == Choice::PAPER)    && (choice2 == Choice::ROCK))
           || ((choice1 == Choice::SCISSORS) && (choice2 == Choice::PAPER));
}

/**
 * Convert a choice to a character.
 * @param choice the Choice enum to convert.
 * @return the character.
 */
inline char to_char(const Choice choice)
{
    return CHOICE_CHARS[static_cast<int>(choice)];
}

/**
 * Convert a choice to a string.
 * @param choice the Choice enum to convert.
 * @return the string.
 */
inline string to_string(const Choice choice)
{
    return CHOICE_STRINGS[static_cast<int>(choice)];
}

/**
 * Convert a character to a choice.
 * @param ch the character to convert.
 * @return the choice.
 */
inline Choice to_choice(const char ch)
{
    switch (toupper(ch))
    {
        case 'R': return Choice::ROCK;
        case 'P': return Choice::PAPER;
        case 'S': return Choice::SCISSORS;

        default:  return Choice::NONE;
    }
}

/**
 * Overloaded output stream operator << for a choice.
 * @param os the output stream.
 * @param choice the choice to output.
 * @return the output stream.
 */
inline ostream& operator <<(ostream& os, const Choice& choice)
{
    os << to_string(choice);
    return os;
}

#endif /* CHOICE_H_ */
