#ifndef RPS_H
#define RPS_H

#include <QMainWindow>

#include "Engine.h"
#include "GUIDisplay.h"
#include "Choice.h"
#include "Algorithm.h"
#include "Human.h"
#include "Computer.h"
#include "Who.h"

QT_BEGIN_NAMESPACE
namespace Ui {
class RPS;
}
QT_END_NAMESPACE

class RPS : public QMainWindow
{
    Q_OBJECT

public:
    RPS(QWidget *parent = nullptr);
    ~RPS();

    /**
     * Show the number of the current round.
     */
    void show_round(const int round) const;

    /**
     * Show the start of a game.
     */
    void show_start_game();

    /**
     * Show the end of a game.
     */
    void show_end_game();

    /**
     * Update the UI.
     * @param human_choice the human's choice.
     * @param computer_choice the computer's choice.
     * @param predicted_choice the predicted choice.
     * @param winner the winner of the round.
     * @param human_wins the number of human wins.
     * @param computer_wins the number of computer wins.
     * @param ties the number of ties.
     */
    void update_UI(const Choice human_choice,
                   const Choice computer_choice,
                   const Choice predicted_choice,
                   const Who winner,
                   const int human_wins,
                   const int computer_wins,
                   const int ties);

    /**
     * Show the end of a game.
     * @param human_wins the number of human wins.
     * @param computer_wins the number of computer wins.
     * @param ties the number of ties.
     * @param max_rounds the max number of rounds per game.
     */
    // void show_game_end(const int human_wins,
    //                    const int computer_wins,
    //                    const int ties,
    //                    const int max_rounds) const;

private slots:
    void on_start_button_clicked();
    void on_rock_button_clicked();
    void on_paper_button_clicked();
    void on_scissors_button_clicked();


private:
    Ui::RPS *ui;

    int smart_index;   // menu index of smart algorithm
    int random_index;  // menu index of random algorithm

    Engine  *engine;   // game engine
    Display *display;  // game display

    Human    *human;     // human player
    Computer *computer;  // computer player

    Algorithm *human_algorithm;     // human's algorihm
    Algorithm *computer_algorithm;  // computer's algorihm

    /**
     * Initialize UI components.
     */
    void initialize_UI();

    /**
     * Do one round with the human's choice.
     * @param the human's choice.
     */
    void do_round(Choice human_choice);
};

#endif // RPS_H
