#ifndef GOOBER_H_
#define GOOBER_H_

#include <string>
#include <map>
#include "ChoiceAlgorithm.h"
#include "RandomChoice.h"
#include "Choice.h"

using namespace std;

const string FREQUENCY_FILE_NAME = "freq.txt";
const int    HISTORY_LENGTH      = 5;

class SmartChoice : public ChoiceAlgorithm
{
public:
    /**
     * Constructor.
     */
    SmartChoice();

    /**
     * Destructor.
     */
    ~SmartChoice();

    /**
     * Override: Make a choice after predicting the human's choice.
     */
    Choice makeChoice();

    Choice predictHumanChoice();

private:
    int history_length;            // length of history string
    string history;                // recent history of players' choices
    string filename;               // frequency file name
    map<string, int> frequencies;  // frequencies of history strings
                                   //   to predict the other's next choice
    RandomChoice randomChooser;    // for when we can't predict

    /**
     * Get the computer player's previous choice.
     * @return the previous choice.
     */
    Choice computerPrevChoice() const;

    /**
     * Get the human player's previous choice.
     * @return the previous choice.
     */
    Choice humanPrevChoice() const;

    /**
     * Append to the recent history, up to MAX_HISTORY length.
     * @param obj a chosen object.
     */
    void appendHistory(const Choice choice);

    /**
     * Update the frequency of the history string.
     */
    void updateFrequency();

    /**
     * Get the frequency of the given history string.
     * @param history the history string.
     * @return the frequency.
     */
    int getFrequency(const string history) const;

    /**
     * Predict the human player's choice for this round.
     */
    Choice predictHumanChoice() const;

    /**
     * Read the frequencies file.
     */
    void readFrequencies();

    /**
     * Write the frequencies file.
     */
    void writeFrequencies();
};

#endif /* GOOBER_H_ */
